// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import qs from "qs";
import { createSpan } from "../util/tracing";
import { CredentialUnavailableError } from "../client/errors";
import { IdentityClient } from "../client/identityClient";
import { SpanStatusCode } from "@azure/core-tracing";
import { credentialLogger, formatSuccess, formatError } from "../util/logging";
import { getIdentityTokenEndpointSuffix } from "../util/identityTokenEndpoint";
import { checkTenantId } from "../util/checkTenantId";
import { processMultiTenantRequest } from "../util/validateMultiTenant";
const logger = credentialLogger("AuthorizationCodeCredential");
/**
 * Enables authentication to Azure Active Directory using an authorization code
 * that was obtained through the authorization code flow, described in more detail
 * in the Azure Active Directory documentation:
 *
 * https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-oauth2-auth-code-flow
 */
export class AuthorizationCodeCredential {
    /**
     * @hidden
     * @internal
     */
    constructor(tenantId, clientId, clientSecretOrAuthorizationCode, authorizationCodeOrRedirectUri, redirectUriOrOptions, options) {
        this.lastTokenResponse = null;
        checkTenantId(logger, tenantId);
        this.clientId = clientId;
        this.tenantId = tenantId;
        if (typeof redirectUriOrOptions === "string") {
            // the clientId+clientSecret constructor
            this.clientSecret = clientSecretOrAuthorizationCode;
            this.authorizationCode = authorizationCodeOrRedirectUri;
            this.redirectUri = redirectUriOrOptions;
            // options okay
        }
        else {
            // clientId only
            this.clientSecret = undefined;
            this.authorizationCode = clientSecretOrAuthorizationCode;
            this.redirectUri = authorizationCodeOrRedirectUri;
            options = redirectUriOrOptions;
        }
        this.allowMultiTenantAuthentication = options === null || options === void 0 ? void 0 : options.allowMultiTenantAuthentication;
        this.identityClient = new IdentityClient(options);
    }
    /**
     * Authenticates with Azure Active Directory and returns an access token if successful.
     * If authentication fails, a {@link CredentialUnavailableError} will be thrown with the details of the failure.
     *
     * @param scopes - The list of scopes for which the token will have access.
     * @param options - The options used to configure any requests this
     *                TokenCredential implementation might make.
     */
    async getToken(scopes, options) {
        var _a, _b;
        const tenantId = processMultiTenantRequest(this.tenantId, this.allowMultiTenantAuthentication, options) ||
            this.tenantId;
        const { span, updatedOptions } = createSpan("AuthorizationCodeCredential-getToken", options);
        try {
            let tokenResponse = null;
            let scopeString = typeof scopes === "string" ? scopes : scopes.join(" ");
            if (scopeString.indexOf("offline_access") < 0) {
                scopeString += " offline_access";
            }
            // Try to use the refresh token first
            if (this.lastTokenResponse && this.lastTokenResponse.refreshToken) {
                tokenResponse = await this.identityClient.refreshAccessToken(tenantId, this.clientId, scopeString, this.lastTokenResponse.refreshToken, this.clientSecret, undefined, updatedOptions);
            }
            if (tokenResponse === null) {
                const urlSuffix = getIdentityTokenEndpointSuffix(tenantId);
                const webResource = this.identityClient.createWebResource({
                    url: `${this.identityClient.authorityHost}/${tenantId}/${urlSuffix}`,
                    method: "POST",
                    disableJsonStringifyOnBody: true,
                    deserializationMapper: undefined,
                    body: qs.stringify({
                        client_id: this.clientId,
                        grant_type: "authorization_code",
                        scope: scopeString,
                        code: this.authorizationCode,
                        redirect_uri: this.redirectUri,
                        client_secret: this.clientSecret
                    }),
                    headers: {
                        Accept: "application/json",
                        "Content-Type": "application/x-www-form-urlencoded"
                    },
                    abortSignal: options && options.abortSignal,
                    spanOptions: (_a = updatedOptions === null || updatedOptions === void 0 ? void 0 : updatedOptions.tracingOptions) === null || _a === void 0 ? void 0 : _a.spanOptions,
                    tracingContext: (_b = updatedOptions === null || updatedOptions === void 0 ? void 0 : updatedOptions.tracingOptions) === null || _b === void 0 ? void 0 : _b.tracingContext
                });
                tokenResponse = await this.identityClient.sendTokenRequest(webResource);
            }
            this.lastTokenResponse = tokenResponse;
            logger.getToken.info(formatSuccess(scopes));
            const token = tokenResponse && tokenResponse.accessToken;
            if (!token) {
                throw new CredentialUnavailableError("Failed to retrieve a valid token");
            }
            return token;
        }
        catch (err) {
            span.setStatus({
                code: SpanStatusCode.ERROR,
                message: err.message
            });
            logger.getToken.info(formatError(scopes, err));
            throw err;
        }
        finally {
            span.end();
        }
    }
}
//# sourceMappingURL=authorizationCodeCredential.js.map