// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { delay, RestError } from "@azure/core-http";
import { SpanStatusCode } from "@azure/core-tracing";
import { AuthenticationError } from "../../client/errors";
import { credentialLogger } from "../../util/logging";
import { createSpan } from "../../util/tracing";
import { imdsApiVersion, imdsEndpoint } from "./constants";
import { msiGenericGetToken } from "./utils";
const logger = credentialLogger("ManagedIdentityCredential - IMDS");
function expiresInParser(requestBody) {
    if (requestBody.expires_on) {
        // Use the expires_on timestamp if it's available
        const expires = +requestBody.expires_on * 1000;
        logger.info(`IMDS using expires_on: ${expires} (original value: ${requestBody.expires_on})`);
        return expires;
    }
    else {
        // If these aren't possible, use expires_in and calculate a timestamp
        const expires = Date.now() + requestBody.expires_in * 1000;
        logger.info(`IMDS using expires_in: ${expires} (original value: ${requestBody.expires_in})`);
        return expires;
    }
}
function prepareRequestOptions(resource, clientId) {
    var _a;
    const queryParameters = {
        resource,
        "api-version": imdsApiVersion
    };
    if (clientId) {
        queryParameters.client_id = clientId;
    }
    return {
        url: (_a = process.env.AZURE_POD_IDENTITY_TOKEN_URL) !== null && _a !== void 0 ? _a : imdsEndpoint,
        method: "GET",
        queryParameters,
        headers: {
            Accept: "application/json",
            Metadata: true
        }
    };
}
// 800ms -> 1600ms -> 3200ms
export const imdsMsiRetryConfig = {
    maxRetries: 3,
    startDelayInMs: 800,
    intervalIncrement: 2
};
export const imdsMsi = {
    async isAvailable(identityClient, resource, clientId, getTokenOptions) {
        var _a, _b, _c;
        const { span, updatedOptions } = createSpan("ManagedIdentityCredential-pingImdsEndpoint", getTokenOptions);
        // if the PodIdenityEndpoint environment variable was set no need to probe the endpoint, it can be assumed to exist
        if (process.env.AZURE_POD_IDENTITY_TOKEN_URL) {
            return true;
        }
        const request = prepareRequestOptions(resource, clientId);
        // This will always be populated, but let's make TypeScript happy
        if (request.headers) {
            // Remove the Metadata header to invoke a request error from
            // IMDS endpoint
            delete request.headers.Metadata;
        }
        request.spanOptions = (_a = updatedOptions === null || updatedOptions === void 0 ? void 0 : updatedOptions.tracingOptions) === null || _a === void 0 ? void 0 : _a.spanOptions;
        request.tracingContext = (_b = updatedOptions === null || updatedOptions === void 0 ? void 0 : updatedOptions.tracingOptions) === null || _b === void 0 ? void 0 : _b.tracingContext;
        try {
            // Create a request with a timeout since we expect that
            // not having a "Metadata" header should cause an error to be
            // returned quickly from the endpoint, proving its availability.
            const webResource = identityClient.createWebResource(request);
            // In Kubernetes pods, node-fetch (used by core-http) takes longer than 2 seconds to begin sending the network request,
            // So smaller timeouts will cause this credential to be immediately aborted.
            // This won't be a problem once we move Identity to core-rest-pipeline.
            webResource.timeout = ((_c = updatedOptions === null || updatedOptions === void 0 ? void 0 : updatedOptions.requestOptions) === null || _c === void 0 ? void 0 : _c.timeout) || 3000;
            try {
                await identityClient.sendRequest(webResource);
            }
            catch (err) {
                if ((err.name === "RestError" && err.code === RestError.REQUEST_SEND_ERROR) ||
                    err.name === "AbortError" ||
                    err.code === "ECONNREFUSED" || // connection refused
                    err.code === "EHOSTDOWN" // host is down
                ) {
                    // If the request failed, or NodeJS was unable to establish a connection,
                    // or the host was down, we'll assume the IMDS endpoint isn't available.
                    logger.info(`The Azure IMDS endpoint is unavailable`);
                    span.setStatus({
                        code: SpanStatusCode.ERROR,
                        message: err.message
                    });
                    return false;
                }
            }
            // If we received any response, the endpoint is available
            logger.info(`The Azure IMDS endpoint is available`);
            // IMDS MSI available!
            return true;
        }
        catch (err) {
            // createWebResource failed.
            // This error should bubble up to the user.
            logger.info(`Error when creating the WebResource for the IMDS endpoint: ${err.message}`);
            span.setStatus({
                code: SpanStatusCode.ERROR,
                message: err.message
            });
            throw err;
        }
        finally {
            span.end();
        }
    },
    async getToken(identityClient, resource, clientId, getTokenOptions = {}) {
        logger.info(`Using the IMDS endpoint coming form the environment variable MSI_ENDPOINT=${process.env.MSI_ENDPOINT}, and using the cloud shell to proceed with the authentication.`);
        let nextDelayInMs = imdsMsiRetryConfig.startDelayInMs;
        for (let retries = 0; retries < imdsMsiRetryConfig.maxRetries; retries++) {
            try {
                return await msiGenericGetToken(identityClient, prepareRequestOptions(resource, clientId), expiresInParser, getTokenOptions);
            }
            catch (error) {
                if (error.statusCode === 404) {
                    await delay(nextDelayInMs);
                    nextDelayInMs *= imdsMsiRetryConfig.intervalIncrement;
                    continue;
                }
                throw error;
            }
        }
        throw new AuthenticationError(404, `Failed to retrieve IMDS token after ${imdsMsiRetryConfig.maxRetries} retries.`);
    }
};
//# sourceMappingURL=imdsMsi.js.map