// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { readFileSync } from "fs";
import { createHash } from "crypto";
import { MsalNode } from "./nodeCommon";
import { formatError } from "../../util/logging";
/**
 * MSAL client certificate client. Calls to MSAL's confidential application's `acquireTokenByClientCredential` during `doGetToken`.
 * @internal
 */
export class MsalClientCertificate extends MsalNode {
    constructor(options) {
        super(options);
        this.requiresConfidential = true;
        this.sendCertificateChain = options.sendCertificateChain;
        const parts = this.parseCertificate(options.certificatePath);
        this.msalConfig.auth.clientCertificate = {
            thumbprint: parts.thumbprint,
            privateKey: parts.certificateContents,
            x5c: parts.x5c
        };
    }
    parseCertificate(certificatePath) {
        const certificateParts = {};
        certificateParts.certificateContents = readFileSync(certificatePath, "utf8");
        if (this.sendCertificateChain) {
            certificateParts.x5c = certificateParts.certificateContents;
        }
        const certificatePattern = /(-+BEGIN CERTIFICATE-+)(\n\r?|\r\n?)([A-Za-z0-9+/\n\r]+=*)(\n\r?|\r\n?)(-+END CERTIFICATE-+)/g;
        const publicKeys = [];
        // Match all possible certificates, in the order they are in the file. These will form the chain that is used for x5c
        let match;
        do {
            match = certificatePattern.exec(certificateParts.certificateContents);
            if (match) {
                publicKeys.push(match[3]);
            }
        } while (match);
        if (publicKeys.length === 0) {
            const error = new Error("The file at the specified path does not contain a PEM-encoded certificate.");
            this.logger.info(formatError("", error));
            throw error;
        }
        certificateParts.thumbprint = createHash("sha1")
            .update(Buffer.from(publicKeys[0], "base64"))
            .digest("hex")
            .toUpperCase();
        return certificateParts;
    }
    async doGetToken(scopes, options = {}) {
        try {
            const result = await this.confidentialApp.acquireTokenByClientCredential({
                scopes,
                correlationId: options.correlationId,
                azureRegion: this.azureRegion,
                authority: options.authority
            });
            // Even though we're providing the same default in memory persistence cache that we use for DeviceCodeCredential,
            // The Client Credential flow does not return the account information from the authentication service,
            // so each time getToken gets called, we will have to acquire a new token through the service.
            return this.handleResult(scopes, this.clientId, result || undefined);
        }
        catch (err) {
            throw this.handleError(scopes, err, options);
        }
    }
}
//# sourceMappingURL=msalClientCertificate.js.map